// -*- C++ -*-
#include "Rivet/Analysis.hh"
#include "Rivet/Projections/FinalState.hh"
#include "Rivet/Projections/IdentifiedFinalState.hh"
#include "Rivet/Projections/DressedLeptons.hh"
#include "Rivet/Projections/FastJets.hh"
#include "Rivet/Projections/VetoedFinalState.hh"
#include "Rivet/Projections/ChargedFinalState.hh"

namespace Rivet {

  class ATLAS_2017_I1589844 : public Analysis {
  public:

    /// @name Constructors etc.
    //@{

    /// Constructors
    ATLAS_2017_I1589844(std::string name="ATLAS_2017_I1589844") 
      : Analysis(name)
    {
      _mode = 1; // pick electron channel by default
      setNeedsCrossSection(true);
    }

    //@}

  public:

    /// @name Analysis methods
    //@{

    /// Book histograms and initialise projections before the run
    void init() {

      FinalState fs;

      Cut cuts = (Cuts::pT > 25*GeV) && ( (Cuts::abseta <= 1.37) || ((Cuts::abseta >= 1.52) && (Cuts::abseta < 2.47)) );
      if (!_mode)  cuts = (Cuts::pT > 25*GeV) && (Cuts::abseta < 2.4); // muon channel version

      IdentifiedFinalState bareleptons(fs);
      bareleptons.acceptIdPair(_mode? PID::ELECTRON : PID::MUON);

      DressedLeptons leptons(fs, bareleptons, 0.1, cuts, true, true);
      addProjection(leptons, "leptons");

      ChargedFinalState cfs(-2.5, 2.5, 0.4*GeV);
      VetoedFinalState jet_fs(cfs);
      jet_fs.addVetoOnThisFinalState(leptons);
      FastJets jetpro04(jet_fs, FastJets::KT, 0.4);
      addProjection(jetpro04, "Kt04Jets");
      FastJets jetpro10(jet_fs, FastJets::KT, 1.0);
      addProjection(jetpro10, "Kt10Jets");

      VetoedFinalState jet_fs_all( FinalState((Cuts::abseta < 2.5) && (Cuts::pT > 0.4*GeV)) );
      jet_fs_all.addVetoOnThisFinalState(leptons);
      FastJets jetpro04_all(jet_fs_all, FastJets::KT, 0.4);
      jetpro04_all.useInvisibles();
      addProjection(jetpro04_all, "Kt04Jets_all");
      FastJets jetpro10_all(jet_fs_all, FastJets::KT, 1.0);
      jetpro10_all.useInvisibles();
      addProjection(jetpro10_all, "Kt10Jets_all");

      _ndij = 8;
      //Histograms with data binning
      for(size_t i = 0; i < _ndij; ++i) {
        string label = "d" + to_str(i) + "_kT4";
        _h[label] = bookHisto1D(i + 1, 1, _mode + 1);
        _h[label + "_all"] = bookHisto1D(i + 1, 1, _mode + 5);
        label = "d" + to_str(i) + "_kT10";
        _h[label] = bookHisto1D(i + 1, 1, _mode + 3);
        _h[label + "_all"] = bookHisto1D(i + 1, 1, _mode + 7);
      }
    }

	  //==========================================================================================


    /// Perform the per-event analysis
    void analyze(const Event& e) {

      
      //---------------------------
      const double weight = e.weight();

      // -- check we have a Z candidate: 
      const vector<DressedLepton> &leptons = applyProjection<DressedLeptons>(e, "leptons").dressedLeptons();
      if (leptons.size() != 2)  vetoEvent;
      if (leptons[0].threeCharge() * leptons[1].threeCharge() > 0)  vetoEvent;
      double dilepton_mass = (leptons[0].momentum() + leptons[1].momentum()).mass();
      if ( !(71*GeV < dilepton_mass && dilepton_mass < 111*GeV) )  vetoEvent;
      
      
      //---------------------------
      //------------- get kT splitting scales
      // charged-only particles
      const FastJets& jetpro04 = applyProjection<FastJets>(e, "Kt04Jets");
      const shared_ptr<fastjet::ClusterSequence> seq04 = jetpro04.clusterSeq();
      for (size_t i = 0; i < min(_ndij, (size_t)seq04->n_particles()); ++i) {
        double dij = sqrt(seq04->exclusive_dmerge_max(i)) * GeV;
        if (dij > 0.0) {
          string label = "d" + to_str(i) + "_kT4";
          _h[label]->fill(dij, weight);
        }
      }
      const FastJets& jetpro10 = applyProjection<FastJets>(e, "Kt10Jets");
      const shared_ptr<fastjet::ClusterSequence> seq10 = jetpro10.clusterSeq();
      for (size_t i = 0; i < min(_ndij, (size_t)seq10->n_particles()); ++i) {
        double dij = sqrt(seq10->exclusive_dmerge_max(i)) * GeV;
        if (dij > 0.0) {
          string label = "d" + to_str(i) + "_kT10";
          _h[label]->fill(dij, weight);
        }
      }

      // all particles
      const FastJets& jetpro04_all = applyProjection<FastJets>(e, "Kt04Jets_all");
      const shared_ptr<fastjet::ClusterSequence> seq04_all = jetpro04_all.clusterSeq();
      for (size_t i = 0; i < min(_ndij, (size_t)seq04_all->n_particles()); ++i) {
        double dij = sqrt(seq04_all->exclusive_dmerge_max(i)) * GeV;
        if (dij > 0.0) {
          string label = "d" + to_str(i) + "_kT4_all";
          _h[label]->fill(dij, weight);
        }
      }
      const FastJets& jetpro10_all = applyProjection<FastJets>(e, "Kt10Jets_all");
      const shared_ptr<fastjet::ClusterSequence> seq10_all = jetpro10_all.clusterSeq();
      for (size_t i = 0; i < min(_ndij, (size_t)seq10_all->n_particles()); ++i) {
        double dij = sqrt(seq10_all->exclusive_dmerge_max(i)) * GeV;
        if (dij > 0.0) {
          string label = "d" + to_str(i) + "_kT10_all";
          _h[label]->fill(dij, weight);
        }
      }

    } // end of analysis loop


    /// Normalise histograms etc., after the run
    void finalize() {
      
      const double sf = crossSectionPerEvent();
      for(map<string, Histo1DPtr>::iterator it = _h.begin(); it != _h.end(); ++it) {
        scale(it->second, sf);
      }
    }
    //@}
    
    
  protected:
	  
    // Data members like post-cuts event weight counters go here
    size_t _mode;
    size_t _ndij;
    
  private:

    map<string, Histo1DPtr> _h;

  };


  class ATLAS_2017_I1589844_EL : public ATLAS_2017_I1589844 {
  public:
    ATLAS_2017_I1589844_EL() : ATLAS_2017_I1589844("ATLAS_2017_I1589844_EL")
    {
      _mode = 1;
    }
  };

  class ATLAS_2017_I1589844_MU : public ATLAS_2017_I1589844 {
  public:
    ATLAS_2017_I1589844_MU() : ATLAS_2017_I1589844("ATLAS_2017_I1589844_MU")
    {
      _mode = 0;
    }
  };


  // The hook for the plugin system
  DECLARE_RIVET_PLUGIN(ATLAS_2017_I1589844);
  DECLARE_RIVET_PLUGIN(ATLAS_2017_I1589844_EL);
  DECLARE_RIVET_PLUGIN(ATLAS_2017_I1589844_MU);

} 

